# assign.py -- test LRU cache assignments

# Copyright 2004 Evan Prodromou <evan@bad.dynu.ca>
# Licensed under the Academic Free License 2.1

# arch-tag: LRU cache assignment tests

"""Test module for basic assignment in LRUCache class"""

import unittest
import lrucache

CACHE_SIZE = 32

class AssignmentTestCase(unittest.TestCase):
    """Base class for most test cases in this module"""
    
    def setUp(self):
        """Setup for all assignment test cases"""
        self.cache = lrucache.LRUCache(CACHE_SIZE)
	
class IntegerKeyCase(AssignmentTestCase):
    """Assigning a value to an integer key"""
    
    def testassign(self):
        """Assign a value with an integer key"""
        self.cache[0] = 'zero'
        self.assertEqual(self.cache[0], 'zero')

class StringKeyCase(AssignmentTestCase):
    """Assigning a value to an string key"""
    
    def testassign(self):
        """Assign a value with a string key"""        
        self.cache['foo'] = 'bar'
        self.assertEqual(self.cache['foo'], 'bar')

class ReassignmentCase(AssignmentTestCase):
    """Assigning two different values to the same key"""
    
    def testreassign(self):
        """Assign two values to a key"""
        self.cache['foo'] = 'bar'
        self.cache['foo'] = 'baz'
        self.assertEqual(self.cache['foo'], 'baz')

class MultiAssignmentCase(AssignmentTestCase):
    """Assigning two different values to two different keys"""
    
    def testmultiassign(self):
        """Assign two values to two keys"""        
        self.cache['foo'] = 'bar'
        self.cache['baz'] = 'quux'
        self.assert_(self.cache['foo'] == 'bar' and self.cache['baz'] == 'quux')

class CacheFillCase(AssignmentTestCase):
    """Filling but not overflowing the cache"""
    
    def setUp(self):
        """Fill cache to prepare for tests"""        
        AssignmentTestCase.setUp(self)
        for i in range(CACHE_SIZE):
            self.cache[i] = i

    def testpresent(self):
        """All items still in cache"""                
        for i in range(CACHE_SIZE):	
            self.assert_(i in self.cache)

    def testequal(self):
        """All items in cache have expected values"""                
        for i in range(CACHE_SIZE):	
            self.assertEqual(self.cache[i], i)
	    
    def testlen(self):
        """Cache length is cache size"""                        
        self.assertEqual(len(self.cache), CACHE_SIZE)
        
class CacheOverflowCase(AssignmentTestCase):
    """overflowing the cache"""
    
    def setUp(self):
        """Fill cache, then overflow by one"""                        
        AssignmentTestCase.setUp(self)
        for i in range(CACHE_SIZE + 1):
            self.cache[i] = i

    def testlru(self):
        """LRU value is no longer in cache"""
        self.assert_(0 not in self.cache)

    def testother(self):
        """Non-LRU values are still in cache"""
        for i in range(CACHE_SIZE):
            self.assert_((i + 1) in self.cache)

    def testsize(self):
        """Size is (still) correct"""                                        
        self.assertEqual(self.cache.size, CACHE_SIZE)

    def testlen(self):
        """Cache length is still cache size"""
        self.assertEqual(len(self.cache), CACHE_SIZE)
	
if __name__ == "__main__":
    unittest.main()
