# contain.py -- test LRU cache containment check

# Copyright 2004 Evan Prodromou <evan@bad.dynu.ca>
# Licensed under the Academic Free License 2.1

# arch-tag: LRU cache containment tests

"""Test cases for containment (is x in cache?)"""

import unittest
import lrucache

CACHE_SIZE = 32

class ContainTestCase(unittest.TestCase):
    """Superclass for most containment test cases"""
    
    def setUp(self):
        """Create new cache to play with"""
        self.cache = lrucache.LRUCache(CACHE_SIZE)

class EmptyCase(ContainTestCase):
    """Empty cache should contain nothing"""
    
    def testint(self):
        """Integer key not in empty cache"""
        self.assert_(0 not in self.cache)

    def teststring(self):
        """String key not in empty cache"""        
        self.assert_('foo' not in self.cache)
	
class IntegerKeyCase(ContainTestCase):
    """Containment of an integer key"""

    def setUp(self):
        """Initialize by assigning integer key"""        
        ContainTestCase.setUp(self)
        self.cache[0] = 'zero'
        
    def testkey(self):
        """Integer key is contained after assignment"""
        self.assert_(0 in self.cache)
        
    def testvalue(self):
        """Value is not contained after assignment of int key"""
        self.assert_('zero' not in self.cache)
	
class StringKeyCase(ContainTestCase):
    """Containment of a string key"""
    
    def setUp(self):
        """Initialize by assigning string key"""
        ContainTestCase.setUp(self)
        self.cache['foo'] = 'bar'
        
    def testkey(self):
        """String key is contained after assignment"""
        self.assert_('foo' in self.cache)
            
    def testvalue(self):
        """Value is not contained after assignment of string key"""
        self.assert_('bar' not in self.cache)

class DeletionCase(ContainTestCase):
    """Containment after deletion"""
    
    def testdelete(self):
        """Key not contained after deletion"""
        self.cache['foo'] = 'bar'
        del self.cache['foo']
        self.assert_('foo' not in self.cache)

class MultiContainCase(ContainTestCase):
    """Containment of multiple keys"""

    def testmultiassign(self):
        """Contain both keys after two key assignments"""
        self.cache['foo'] = 'bar'
        self.cache['baz'] = 'quux'
        self.assert_('foo' in self.cache and 'baz' in self.cache)

class CacheFillCase(ContainTestCase):
    """Containment after cache is full"""
    
    def setUp(self):
        """Initialize by filling cache"""
        ContainTestCase.setUp(self)
        for i in range(CACHE_SIZE):
            self.cache[i] = i

    def testpresent(self):
        """All values contained after filling cache"""
        for i in range(CACHE_SIZE):	
            self.assert_(i in self.cache)
            
class CacheOverflowCase(ContainTestCase):
    """Containment after cache overflow"""
    
    def setUp(self):
        """Overflow cache by one"""
        ContainTestCase.setUp(self)
        for i in range(CACHE_SIZE + 1):
            self.cache[i] = i

    def testlru(self):
        """LRU key not contained after overflow"""
        self.assert_(0 not in self.cache)

    def testother(self):
        """All non-LRU keys contained after overflow"""
        for i in range(CACHE_SIZE):
            self.assert_((i + 1) in self.cache)

if __name__ == "__main__":
    unittest.main()
