# delete.py -- test LRU cache deletion

# Copyright 2004 Evan Prodromou <evan@bad.dynu.ca>
# Licensed under the Academic Free License 2.1

# arch-tag: LRU cache deletion tests

"""Test cases for deleting items manually out of the cache."""

import unittest
import lrucache

CACHE_SIZE = 32

class DeleteTestCase(unittest.TestCase):
    """Base class for sharing code between deletion test cases."""
    
    def setUp(self):
        """Initialize a fresh cache to play with for each test case."""
        self.cache = lrucache.LRUCache(CACHE_SIZE)

    def delete_wrap(self, key):
        """wrapper for del command, since assertRaises
        only takes a callable
        """
        del self.cache[key]
	
class EmptyCase(DeleteTestCase):
    """Delete in an empty cache"""
    
    def testdelete(self):
        """CacheKeyError when deleting from empty cache"""
        self.assertRaises(lrucache.CacheKeyError, self.delete_wrap, 0)

class IntegerKeyCase(DeleteTestCase):
    """Delete an integer key from cache"""
    
    def testdelete(self):
        """Delete an integer key from cache"""
        self.cache[0] = 'zero'
        del self.cache[0]
        self.assert_(0 not in self.cache)

class StringKeyCase(DeleteTestCase):
    """Delete a string key from cache"""
    
    def testassign(self):
        """Delete a string key from cache"""
        self.cache['foo'] = 'bar'
        del self.cache['foo']
        self.assert_('foo' not in self.cache)

class BadKeyCase(DeleteTestCase):
    """Delete a non-present key from cache"""
    
    def testdelete(self):
        """CacheKeyError when delete a non-present key from cache"""
        self.cache['foo'] = 'bar'
        self.assertRaises(lrucache.CacheKeyError, self.delete_wrap, 'baz')

class DoubleDeleteCase(DeleteTestCase):
    """Delete a key twice from cache"""
    
    def testdelete(self):
        """CacheKeyError when delete a key twice from cache"""
        self.cache['foo'] = 'bar'
        del self.cache['foo']
        self.assertRaises(lrucache.CacheKeyError, self.delete_wrap, 'foo')
	
class MultiDeleteCase(DeleteTestCase):
    """Base case for deleting with multiple keys"""
    
    def setUp(self):
        """Put two keys in cache"""
        DeleteTestCase.setUp(self)
        self.cache['foo'] = 'bar'
        self.cache['baz'] = 'quux'

class DeleteBothMultiCase(MultiDeleteCase):
    """Delete both keys from a multi-key cache"""
    
    def setUp(self):
        """Delete both keys from a multi-key cache"""        
        MultiDeleteCase.setUp(self)
        del self.cache['foo']
        del self.cache['baz']

    def testcontain(self):
        """Both keys gone after delete both keys"""
        self.assert_('foo' not in self.cache and 'baz' not in self.cache)

    def testlen(self):
        """Cache length zero after delete both keys"""        
        self.assertEqual(len(self.cache), 0)

class DeleteOneMultiCase(MultiDeleteCase):
    """Delete one key from a multi-key cache"""
    
    def setUp(self):
        """Delete one key from a multi-key cache"""        
        MultiDeleteCase.setUp(self)
        del self.cache['foo']

    def testcontain(self):
        """One key gone, one key there after delete one key"""
        self.assert_('foo' not in self.cache and 'baz' in self.cache)

    def testlen(self):
        """Length is 1 after delete one of two keys"""        
        self.assertEqual(len(self.cache), 1)
	
class CacheOverflowCase(DeleteTestCase):
    """Delete key pushed out by overflow"""
    
    def setUp(self):
        """Init by forcing overflow"""        
        DeleteTestCase.setUp(self)
        for i in range(CACHE_SIZE + 1):
            self.cache[i] = i

    def testzero(self):
        """CacheKeyError when deleting overflowed key"""                
        self.assertRaises(lrucache.CacheKeyError, self.delete_wrap, 0)
	
if __name__ == "__main__":
    unittest.main()
