# iterator.py -- tests for cache iterators

# Copyright 2004 Evan Prodromou <evan@bad.dynu.ca>
# Licensed under the Academic Free License 2.1

# arch-tag: LRU cache iterator tests

"""Test cases for iterators over a cache"""

import unittest
import lrucache

CACHE_SIZE = 12

class IteratorTestCase(unittest.TestCase):
    """Base class for mtime test cases"""
    
    def setUp(self):
        """Create a cache"""
        self.cache = lrucache.LRUCache(CACHE_SIZE)

class EmptyCacheCase(IteratorTestCase):
    """No iterations over an empty cache"""
    
    def testempty(self):
        """Does not iterate over an empty cache"""
        for item in self.cache:
            # shouldn't get here
            self.fail(self.__doc__)

class ExampleCase(IteratorTestCase):
    """Iterator test case with example data"""

    months = (('January', 31),
              ('February', 28),
              ('March', 31),
              ('April', 30),
              ('May', 31),
              ('June', 30),
              ('July', 31),
              ('August', 31),
              ('September', 30),
              ('October', 31),
              ('November', 30),
              ('December', 31))
    """A tuple-map of months to month length. We use a
    tuple so we can be guaranteed of order."""
    
    def setUp(self):
        """Add example data to cache"""
        IteratorTestCase.setUp(self)
        for month in self.months:
            self.cache[month[0]] = month[1]
            
class SimpleCase(ExampleCase):
    """Iterator iterates over cache"""

    def testsimple(self):
        """Iterator returns added values
        (and only added values)
        """
        # utility dictionary for comparison
        compare = {}
        for month in self.months:
            compare[month[0]] = {'value' : month[1], 'seen' : False}
            
        for key in self.cache:
            self.assert_(type(key) == type("string"))
            self.assert_(key in compare)
            self.assert_(self.cache[key] == compare[key]['value'])
            self.assert_(not compare[key]['seen'])
            compare[key]['seen'] = True

        for key in compare:
            self.assert_(compare[key]['seen'])
            
class DeleteCase(ExampleCase):
    """Iterator doesn't return deleted values"""
    
    def testdelete(self):
        """Iterator doesn't return deleted values"""
        del self.cache['March']
        for key in self.cache:
            self.assertNotEqual(key, 'March')

class OverflowCase(ExampleCase):
    """Iterator doesn't return discarded value after overflow"""
    
    def testoverflow(self):
        """Iterator doesn't return overflowed values"""
        self.cache['Germinal'] = 30
        for key in self.cache:
            self.assertNotEqual(key, 'January')
        
if __name__ == "__main__":
    unittest.main()
