# lru.py -- test that LRU is really LRU

# Copyright 2004 Evan Prodromou <evan@bad.dynu.ca>
# Licensed under the Academic Free License 2.1

# arch-tag: LRU cache ordering tests

"""Test cases for the LRU ordering of the cache"""

import unittest
import lrucache

CACHE_SIZE = 64

class OrderTestCase(unittest.TestCase):
    """Base class for LRU ordering test cases"""
    
    def setUp(self):
        """Create a cache and fill it"""
        self.cache = lrucache.LRUCache(CACHE_SIZE)
        for i in range(CACHE_SIZE):
            self.cache[i] = i
	    
    def testlen(self):
        """Length of cache is configured size"""        
        self.assertEqual(len(self.cache), CACHE_SIZE)

    def testsize(self):
        """Size of cache is configured size"""
        self.assertEqual(self.cache.size, CACHE_SIZE)
	
class SimpleCase(OrderTestCase):
    """Basic cache discard"""
    
    def testsimple(self):
        """Add one key to full cache, oldest is discarded"""                
        self.cache[CACHE_SIZE] = CACHE_SIZE
        self.assert_(0 not in self.cache)

class MultiCase(OrderTestCase):
    """Discard multiple keys from cache"""
    
    def setUp(self):
        """Add two extra keys to a full cache"""
        OrderTestCase.setUp(self)
        self.cache[CACHE_SIZE] = CACHE_SIZE
        self.cache[CACHE_SIZE + 1] = CACHE_SIZE + 1
    
    def testzero(self):
        """Oldest key gone after adding two keys
        to a full cache
        """
        self.assert_(0 not in self.cache)

    def testone(self):
        """Second-oldest key gone after adding
        two keys to a full cache
        """
        self.assert_(1 not in self.cache)

    def testtwo(self):
        """Third-oldest key still present after adding
        two keys to a full cache
        """
        self.assert_(2 in self.cache)
	
class ReadCase(OrderTestCase):
    """Oldest key no longer oldest after read"""

    def setUp(self):
        """Read oldest key then overflow cache"""
        OrderTestCase.setUp(self)
        unused = self.cache[0]
        self.cache[CACHE_SIZE] = CACHE_SIZE
    
    def testzero(self):
        """Oldest key not discarded after read
        then overflow
        """
        self.assert_(0 in self.cache)

    def testone(self):
        """Second-oldest key discarded after 
        oldest key was read, then cache overflow
        """
        self.assert_(1 not in self.cache)

class WriteCase(OrderTestCase):
    """Oldest key no longer oldest after write"""
    
    def setUp(self):
        """Write oldest key then overflow cache"""
        OrderTestCase.setUp(self)
        self.cache[0] = 'foo'
        self.cache[CACHE_SIZE] = CACHE_SIZE
    
    def testzero(self):
        """Oldest key not discarded after write
        then overflow
        """
        self.assert_(0 in self.cache)

    def testone(self):
        """Second-oldest key discarded after 
        oldest key was written, then cache overflow
        """
        self.assert_(1 not in self.cache)

if __name__ == "__main__":
    unittest.main()
