# mtime.py -- tests for LRU mtime check

# Copyright 2004 Evan Prodromou <evan@bad.dynu.ca>
# Licensed under the Academic Free License 2.1

# arch-tag: LRU cache mtime tests

"""Test cases for the mtime method of the cache"""

import unittest
import lrucache
import time

CACHE_SIZE = 64

class MtimeTestCase(unittest.TestCase):
    """Base class for mtime test cases"""
    
    def setUp(self):
        """Create a cache"""
        self.cache = lrucache.LRUCache(CACHE_SIZE)

class EmptyCacheCase(MtimeTestCase):
    """Mtime of any key in empty cache should raise an error"""

    def testempty(self):
        """Mtime raises error for any key on empty cache"""
        self.assertRaises(lrucache.CacheKeyError, self.cache.mtime, 'foo')
        
class SimpleCase(MtimeTestCase):
    """Check mtime of one added key"""

    def testsimple(self):
        """Mtime between zero and now for added key
        """
        self.cache['foo'] = 'bar'
        self.assert_(self.cache.mtime('foo') > 0)
        self.assert_(self.cache.mtime('foo') <= time.time())

class AccurateCase(MtimeTestCase):
    """Mtime should be recording the time of modification"""
    
    def testaccurate(self):
        """Mtime is saving the time of modification"""
        before = time.time()
        self.cache['foo'] = 'bar'
        after = time.time()
        self.assert_(before <= self.cache.mtime('foo'))
        self.assert_(self.cache.mtime('foo') <= after)

class BadKeyCase(MtimeTestCase):
    """Mtime of key not in cache should raise an error"""

    def testbad(self):
        """Mtime of key not in cache raises error"""
        self.cache['foo'] = 'bar'
        self.assertRaises(lrucache.CacheKeyError, self.cache.mtime, 'baz')
        
class MultiCase(MtimeTestCase):
    """Check mtime of multiple added keys"""

    def testvalues(self):
        """Mtime between zero and now for added keys
        """
        self.cache['foo'] = 'bar'
        self.cache[0] = 'zero'
        self.assert_(self.cache.mtime('foo') > 0)
        self.assert_(self.cache.mtime('foo') <= time.time())
        self.assert_(self.cache.mtime(0) > 0)
        self.assert_(self.cache.mtime(0) <= time.time())

class MultiDifferentCase(MtimeTestCase):
    """Mtime should be different for keys added separately"""
    
    def testdifferent(self):
        """Mtime different for keys added at different times"""
        self.cache['foo'] = 'bar'
        time.sleep(1)
        self.cache[0] = 'zero'
        self.assert_(self.cache.mtime(0) > self.cache.mtime('foo'))
        
class ReadCase(MtimeTestCase):
    """Mtime doesn't change after a read"""

    def testread(self):
        """Mtime doesn't change after a read"""
        self.cache['foo'] = 'bar'
        old_time = self.cache.mtime('foo')
        time.sleep(1)
        unused = self.cache['foo']
        self.assertAlmostEqual(old_time, self.cache.mtime('foo'))
        
class WriteCase(MtimeTestCase):
    """Mtime changes after a write"""
    
    def testwrite(self):
        """Mtime changes after a write"""
        self.cache['foo'] = 'bar'
        old_time = self.cache.mtime('foo')
        time.sleep(1)
        self.cache['foo'] = 'baz'
        self.assert_(self.cache.mtime('foo') > old_time)

class OverflowCase(MtimeTestCase):
    """Mtime of key overflowed from cache should raise an error"""

    def testempty(self):
        """Mtime of key overflowed from cache raises error"""
        # fill the cache
        for i in range(CACHE_SIZE):
            self.cache[i] = i
        # overflow
        self.cache[CACHE_SIZE] = 'overflow'
        # 0 is oldest, should have been discarded
        self.assertRaises(lrucache.CacheKeyError, self.cache.mtime, 0)
        
if __name__ == "__main__":
    unittest.main()
